@@ -10,6 +10,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
 import inspect
 import os
 import re
@@ -37,6 +38,9 @@ from networking_ovn.common import exceptions as ovn_exc
 
 DNS_RESOLVER_FILE = "/etc/resolv.conf"
 
+AddrPairsDiff = collections.namedtuple(
+    'AddrPairsDiff', ['added', 'removed', 'changed'])
+
 
 def ovn_name(id):
     # The name of the OVN entry will be neutron-<UUID>
@@ -426,3 +430,34 @@ def is_neutron_dhcp_agent_port(port):
     return (port['device_owner'] == const.DEVICE_OWNER_DHCP and
             (port['device_id'] == const.DEVICE_ID_RESERVED_DHCP_PORT or
              port['device_id'].startswith('dhcp')))
+
+
+def is_gateway_chassis(chassis):
+    """Check if the given chassis is a gateway chassis"""
+    external_ids = getattr(chassis, 'external_ids', {})
+    return ('enable-chassis-as-gw' in external_ids.get(
+        'ovn-cms-options', '').split(','))
+
+
+def get_port_id_from_gwc_row(row):
+    """Return a port_id from gwc row
+
+    The Gateway_Chassis row stores router port_id in
+    the row name attribute:
+
+    <prefix>-<port_id>_<chassis_id>
+
+    :param row: A Gateway_Chassis table row.
+    :returns: String containing router port_id.
+    """
+    return constants.RE_PORT_FROM_GWC.search(row.name).group(2)
+
+
+def compute_address_pairs_diff(ovn_port, neutron_port):
+    """Compute the differences in the allowed_address_pairs field."""
+    ovn_ap = get_allowed_address_pairs_ip_addresses_from_ovn_port(
+        ovn_port)
+    neutron_ap = get_allowed_address_pairs_ip_addresses(neutron_port)
+    added = set(neutron_ap) - set(ovn_ap)
+    removed = set(ovn_ap) - set(neutron_ap)
+    return AddrPairsDiff(added, removed, changed=any(added or removed))
