@@ -12,15 +12,16 @@
 
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
 from neutron_lib import context as n_context
 from neutron_lib import exceptions as n_exc
+from neutron_lib.services.trunk import constants as trunk_consts
 from oslo_config import cfg
 from oslo_log import log
 
 from networking_ovn.common.constants import OVN_ML2_MECH_DRIVER_NAME
 
 from neutron.objects import ports as port_obj
-from neutron.services.trunk import constants as trunk_consts
 from neutron.services.trunk.drivers import base as trunk_base
 from neutron_lib.api.definitions import portbindings
 
@@ -31,7 +32,7 @@ SUPPORTED_INTERFACES = (
 )
 
 SUPPORTED_SEGMENTATION_TYPES = (
-    trunk_consts.VLAN,
+    trunk_consts.SEGMENTATION_TYPE_VLAN,
 )
 
 LOG = log.getLogger(__name__)
@@ -44,20 +45,22 @@ class OVNTrunkHandler(object):
     def _set_sub_ports(self, parent_port, subports):
         txn = self.plugin_driver._nb_ovn.transaction
         context = n_context.get_admin_context()
-        with context.session.begin(subtransactions=True), (
-                txn(check_error=True)) as ovn_txn:
-            for port in subports:
+        for port in subports:
+            with context.session.begin(subtransactions=True), (
+                    txn(check_error=True)) as ovn_txn:
                 self._set_binding_profile(context, port, parent_port, ovn_txn)
 
     def _unset_sub_ports(self, subports):
         txn = self.plugin_driver._nb_ovn.transaction
         context = n_context.get_admin_context()
-        with context.session.begin(subtransactions=True), (
-                txn(check_error=True)) as ovn_txn:
-            for port in subports:
+        for port in subports:
+            with context.session.begin(subtransactions=True), (
+                    txn(check_error=True)) as ovn_txn:
                 self._unset_binding_profile(context, port, ovn_txn)
 
     def _set_binding_profile(self, context, subport, parent_port, ovn_txn):
+        LOG.debug("Setting parent %s for subport %s",
+                  parent_port, subport.port_id)
         db_port = port_obj.Port.get_object(context, id=subport.port_id)
         if not db_port:
             LOG.debug("Port not found while trying to set "
@@ -65,14 +68,20 @@ class OVNTrunkHandler(object):
                       subport.port_id)
             return
         try:
-            db_port.binding.profile.update({
-                'parent_name': parent_port,
-                'tag': subport.segmentation_id})
-            # host + port_id is primary key
-            port_obj.PortBinding.update_object(
-                context, {'profile': db_port.binding.profile},
-                port_id=subport.port_id,
-                host=db_port.binding.host)
+            # NOTE(flaviof): We expect binding's host to be set. Otherwise,
+            # sub-port will not transition from DOWN to ACTIVE.
+            db_port.device_owner = trunk_consts.TRUNK_SUBPORT_OWNER
+            for binding in db_port.bindings:
+                binding.profile['parent_name'] = parent_port
+                binding.profile['tag'] = subport.segmentation_id
+                # host + port_id is primary key
+                port_obj.PortBinding.update_object(
+                    context,
+                    {'profile': binding.profile,
+                     'vif_type': portbindings.VIF_TYPE_OVS},
+                    port_id=subport.port_id,
+                    host=binding.host)
+            db_port.update()
         except n_exc.ObjectNotFound:
             LOG.debug("Port not found while trying to set "
                       "binding_profile: %s", subport.port_id)
@@ -81,8 +90,11 @@ class OVNTrunkHandler(object):
                     lport_name=subport.port_id,
                     parent_name=parent_port,
                     tag=subport.segmentation_id))
+        LOG.debug("Done setting parent %s for subport %s",
+                  parent_port, subport.port_id)
 
     def _unset_binding_profile(self, context, subport, ovn_txn):
+        LOG.debug("Unsetting parent for subport %s", subport.port_id)
         db_port = port_obj.Port.get_object(context, id=subport.port_id)
         if not db_port:
             LOG.debug("Port not found while trying to unset "
@@ -90,20 +102,20 @@ class OVNTrunkHandler(object):
                       subport.port_id)
             return
         try:
-            db_port.binding.profile.pop('tag', None)
-            db_port.binding.profile.pop('parent_name', None)
-            # host + port_id is primary key
-            port_obj.PortBinding.update_object(
-                context,
-                {'profile': db_port.binding.profile,
-                 'vif_type': portbindings.VIF_TYPE_UNBOUND,
-                 'vif_details': '',
-                 'host': ''},
-                port_id=subport.port_id,
-                host=db_port.binding.host)
-            port_obj.PortBindingLevel.delete_objects(
-                context, port_id=subport.port_id,
-                host=db_port.binding.host)
+            db_port.device_owner = ''
+            for binding in db_port.bindings:
+                binding.profile.pop('tag', None)
+                binding.profile.pop('parent_name', None)
+                # host + port_id is primary key
+                port_obj.PortBinding.update_object(
+                    context,
+                    {'profile': binding.profile,
+                     'vif_type': portbindings.VIF_TYPE_UNBOUND},
+                    port_id=subport.port_id,
+                    host=binding.host)
+                port_obj.PortBindingLevel.delete_objects(
+                    context, port_id=subport.port_id, host=binding.host)
+            db_port.update()
         except n_exc.ObjectNotFound:
             LOG.debug("Port not found while trying to unset "
                       "binding_profile: %s", subport.port_id)
@@ -113,11 +125,12 @@ class OVNTrunkHandler(object):
                     parent_name=[],
                     up=False,
                     tag=[]))
+        LOG.debug("Done unsetting parent for subport %s", subport.port_id)
 
     def trunk_created(self, trunk):
         if trunk.sub_ports:
             self._set_sub_ports(trunk.port_id, trunk.sub_ports)
-        trunk.update(status=trunk_consts.ACTIVE_STATUS)
+        trunk.update(status=trunk_consts.TRUNK_ACTIVE_STATUS)
 
     def trunk_deleted(self, trunk):
         if trunk.sub_ports:
@@ -126,12 +139,12 @@ class OVNTrunkHandler(object):
     def subports_added(self, trunk, subports):
         if subports:
             self._set_sub_ports(trunk.port_id, subports)
-        trunk.update(status=trunk_consts.ACTIVE_STATUS)
+        trunk.update(status=trunk_consts.TRUNK_ACTIVE_STATUS)
 
     def subports_deleted(self, trunk, subports):
         if subports:
             self._unset_sub_ports(subports)
-        trunk.update(status=trunk_consts.ACTIVE_STATUS)
+        trunk.update(status=trunk_consts.TRUNK_ACTIVE_STATUS)
 
     def trunk_event(self, resource, event, trunk_plugin, payload):
         if event == events.AFTER_CREATE:
@@ -156,17 +169,17 @@ class OVNTrunkDriver(trunk_base.DriverBase):
         except cfg.NoSuchOptError:
             return False
 
-    @registry.receives(trunk_consts.TRUNK_PLUGIN, [events.AFTER_INIT])
+    @registry.receives(resources.TRUNK_PLUGIN, [events.AFTER_INIT])
     def register(self, resource, event, trigger, payload=None):
         super(OVNTrunkDriver, self).register(
             resource, event, trigger, payload=payload)
         self._handler = OVNTrunkHandler(self.plugin_driver)
         for trunk_event in (events.AFTER_CREATE, events.AFTER_DELETE):
             registry.subscribe(self._handler.trunk_event,
-                               trunk_consts.TRUNK,
+                               resources.TRUNK,
                                trunk_event)
             registry.subscribe(self._handler.subport_event,
-                               trunk_consts.SUBPORTS,
+                               resources.SUBPORTS,
                                trunk_event)
 
     @classmethod
