@@ -86,9 +86,9 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         LOG.debug("Starting OVN-Northbound DB sync process")
 
         ctx = context.get_admin_context()
-        self.migrate_to_port_groups(ctx)
 
         self.sync_address_sets(ctx)
+        self.sync_port_groups(ctx)
         self.sync_networks_ports_and_dhcp_opts(ctx)
         self.sync_port_dns_records(ctx)
         self.sync_acls(ctx)
@@ -167,6 +167,74 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
     def get_address_sets(self):
         return self.ovn_api.get_address_sets()
 
+    def sync_port_groups(self, ctx):
+        """Sync Port Groups between neutron and NB.
+
+        @param ctx: neutron_lib.context
+        @type  ctx: object of type neutron_lib.context.Context
+        """
+        if not self.ovn_api.is_port_groups_supported():
+            return
+
+        neutron_sgs = {}
+        neutron_pgs = set()
+        with ctx.session.begin(subtransactions=True):
+            for sg in self.core_plugin.get_security_groups(ctx):
+                pg_name = utils.ovn_port_group_name(sg['id'])
+                neutron_pgs.add(pg_name)
+                neutron_sgs[pg_name] = sg['id']
+            neutron_pgs.add(const.OVN_DROP_PORT_GROUP_NAME)
+
+        ovn_pgs = set()
+        port_groups = self.ovn_api.db_list_rows('Port_Group').execute() or []
+        for pg in port_groups:
+            ovn_pgs.add(pg.name)
+
+        add_pgs = neutron_pgs.difference(ovn_pgs)
+        remove_pgs = ovn_pgs.difference(neutron_pgs)
+
+        LOG.debug('Port Groups added %d, removed %d',
+                  len(add_pgs), len(remove_pgs))
+
+        if self.mode == SYNC_MODE_REPAIR:
+            LOG.debug('Port-Group-SYNC: transaction started @ %s',
+                      str(datetime.now()))
+            if add_pgs:
+                db_ports = self.core_plugin.get_ports(ctx)
+                ovn_ports = set(p.name for p in
+                                self.ovn_api.lsp_list().execute())
+            with self.ovn_api.transaction(check_error=True) as txn:
+                pg = const.OVN_DROP_PORT_GROUP_NAME
+                # Process default drop port group first
+                if pg in add_pgs:
+                    txn.add(self.ovn_api.pg_add(name=pg, acls=[]))
+                    add_pgs.remove(pg)
+                    # Add ports to the drop port group. Only add those that
+                    # already exists in OVN. The rest will be added during the
+                    # ports sync operation later.
+                    for n_port in db_ports:
+                        if ((n_port['security_groups'] or
+                                n_port['port_security_enabled']) and
+                                n_port['id'] in ovn_ports):
+                            txn.add(self.ovn_api.pg_add_ports(
+                                pg, n_port['id']))
+
+                for pg in add_pgs:
+                    # If it's a security group PG, add the ext id
+                    ext_ids = {const.OVN_SG_EXT_ID_KEY: neutron_sgs[pg]}
+                    txn.add(self.ovn_api.pg_add(name=pg, acls=[],
+                                                external_ids=ext_ids))
+                    # Add the ports belonging to the SG to this port group
+                    for n_port in db_ports:
+                        if (neutron_sgs[pg] in n_port['security_groups'] and
+                                n_port['id'] in ovn_ports):
+                            txn.add(self.ovn_api.pg_add_ports(
+                                pg, n_port['id']))
+                for pg in remove_pgs:
+                    txn.add(self.ovn_api.pg_del(pg))
+            LOG.debug('Port-Group-SYNC: transaction finished @ %s',
+                      str(datetime.now()))
+
     def sync_address_sets(self, ctx):
         """Sync Address Sets between neutron and NB.
 
@@ -174,41 +242,48 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         @type  ctx: object of type neutron_lib.context.Context
         @var   db_ports: List of ports from neutron DB
         """
-        LOG.debug('Address-Set-SYNC: started @ %s' % str(datetime.now()))
-
-        neutron_sgs = {}
-        with ctx.session.begin(subtransactions=True):
-            db_sgs = self.core_plugin.get_security_groups(ctx)
-            db_ports = self.core_plugin.get_ports(ctx)
-
-        for sg in db_sgs:
-            for ip_version in ['ip4', 'ip6']:
-                name = utils.ovn_addrset_name(sg['id'], ip_version)
-                neutron_sgs[name] = {
-                    'name': name, 'addresses': [],
-                    'external_ids': {const.OVN_SG_EXT_ID_KEY: sg['id']}}
-
-        for port in db_ports:
-            sg_ids = utils.get_lsp_security_groups(port)
-            if port.get('fixed_ips') and sg_ids:
-                addresses = acl_utils.acl_port_ips(port)
-                for sg_id in sg_ids:
-                    for ip_version in addresses:
-                        name = utils.ovn_addrset_name(sg_id, ip_version)
-                        neutron_sgs[name]['addresses'].extend(
-                            addresses[ip_version])
+        LOG.debug('Address-Set-SYNC: started @ %s', str(datetime.now()))
 
+        sgnames_to_add = sgnames_to_delete = []
+        sgs_to_update = {}
         nb_sgs = self.get_address_sets()
 
-        sgnames_to_add, sgnames_to_delete, sgs_to_update =\
-            self.compute_address_set_difference(neutron_sgs, nb_sgs)
+        if self.ovn_api.is_port_groups_supported():
+            # If Port Groups are supported, we just need to delete all Address
+            # Sets from NB database.
+            sgnames_to_delete = nb_sgs.keys()
+        else:
+            neutron_sgs = {}
+            with ctx.session.begin(subtransactions=True):
+                db_sgs = self.core_plugin.get_security_groups(ctx)
+                db_ports = self.core_plugin.get_ports(ctx)
+
+            for sg in db_sgs:
+                for ip_version in ['ip4', 'ip6']:
+                    name = utils.ovn_addrset_name(sg['id'], ip_version)
+                    neutron_sgs[name] = {
+                        'name': name, 'addresses': [],
+                        'external_ids': {const.OVN_SG_EXT_ID_KEY: sg['id']}}
+
+            for port in db_ports:
+                sg_ids = utils.get_lsp_security_groups(port)
+                if port.get('fixed_ips') and sg_ids:
+                    addresses = acl_utils.acl_port_ips(port)
+                    for sg_id in sg_ids:
+                        for ip_version in addresses:
+                            name = utils.ovn_addrset_name(sg_id, ip_version)
+                            neutron_sgs[name]['addresses'].extend(
+                                addresses[ip_version])
+
+            sgnames_to_add, sgnames_to_delete, sgs_to_update =\
+                self.compute_address_set_difference(neutron_sgs, nb_sgs)
 
         LOG.debug('Address_Sets added %d, removed %d, updated %d',
                   len(sgnames_to_add), len(sgnames_to_delete),
                   len(sgs_to_update))
 
         if self.mode == SYNC_MODE_REPAIR:
-            LOG.debug('Address-Set-SYNC: transaction started @ %s' %
+            LOG.debug('Address-Set-SYNC: transaction started @ %s',
                       str(datetime.now()))
             with self.ovn_api.transaction(check_error=True) as txn:
                 for sgname in sgnames_to_add:
@@ -218,11 +293,88 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                     txn.add(self.ovn_api.update_address_set(**sg))
                 for sgname in sgnames_to_delete:
                     txn.add(self.ovn_api.delete_address_set(name=sgname))
-            LOG.debug('Address-Set-SYNC: transaction finished @ %s' %
+            LOG.debug('Address-Set-SYNC: transaction finished @ %s',
                       str(datetime.now()))
 
-    def sync_acls(self, ctx):
-        """Sync ACLs between neutron and NB.
+    def _get_acls_from_port_groups(self):
+        ovn_acls = []
+        port_groups = self.ovn_api.db_list_rows('Port_Group').execute()
+        for pg in port_groups:
+            acls = getattr(pg, 'acls', [])
+            for acl in acls:
+                acl_string = {}
+                acl_string['port_group'] = pg.name
+                for acl_key in getattr(acl, "_data", {}):
+                    acl_string[acl_key] = getattr(acl, acl_key)
+                acl_string.pop('meter')
+                acl_string.pop('external_ids')
+                ovn_acls.append(acl_string)
+        return ovn_acls
+
+    def _sync_acls_port_groups(self, ctx):
+        # If Port Groups are supported, the ACLs in the system will equal
+        # the number of SG rules plus the default drop rules as OVN would
+        # allow all traffic by default if those are not added.
+        neutron_acls = []
+        for sgr in self.core_plugin.get_security_group_rules(ctx):
+            pg_name = utils.ovn_port_group_name(sgr['security_group_id'])
+            neutron_acls.append(acl_utils._add_sg_rule_acl_for_port_group(
+                pg_name, sgr, self.ovn_api))
+        neutron_acls += acl_utils.add_acls_for_drop_port_group(
+            const.OVN_DROP_PORT_GROUP_NAME)
+
+        ovn_acls = self._get_acls_from_port_groups()
+
+        # We need to remove also all the ACLs applied to Logical Switches
+        def get_num_acls(ovn_acls):
+            return len([item for sublist in ovn_acls for item in sublist[1]])
+
+        ovn_acls_from_ls = [(row.name, row.acls) for row in (
+            self.ovn_api._tables['Logical_Switch'].rows.values())]
+        num_acls_to_remove_from_ls = get_num_acls(ovn_acls_from_ls)
+
+        # Remove the common ones
+        for na in list(neutron_acls):
+            for ovn_a in ovn_acls:
+                if all(item in na.items() for item in ovn_a.items()):
+                    neutron_acls.remove(na)
+                    ovn_acls.remove(ovn_a)
+                    break
+
+        num_acls_to_add = len(neutron_acls)
+        num_acls_to_remove = len(ovn_acls) + num_acls_to_remove_from_ls
+        if 0 != num_acls_to_add or 0 != num_acls_to_remove:
+            LOG.warning('ACLs-to-be-added %(add)d '
+                        'ACLs-to-be-removed %(remove)d',
+                        {'add': num_acls_to_add,
+                         'remove': num_acls_to_remove})
+
+        if self.mode == SYNC_MODE_REPAIR:
+            with self.ovn_api.transaction(check_error=True) as txn:
+                for acla in neutron_acls:
+                    LOG.warning('ACL found in Neutron but not in '
+                                'OVN DB for port group %s', acla['port_group'])
+                    txn.add(self.ovn_api.pg_acl_add(**acla))
+
+            with self.ovn_api.transaction(check_error=True) as txn:
+                for aclr in ovn_acls:
+                    LOG.warning('ACLs found in OVN DB but not in '
+                                'Neutron for port group %s',
+                                aclr['port_group'])
+                    txn.add(self.ovn_api.pg_acl_del(aclr['port_group'],
+                                                    aclr['direction'],
+                                                    aclr['priority'],
+                                                    aclr['match']))
+                for aclr in ovn_acls_from_ls:
+                    # Remove all the ACLs from any Logical Switch if they have
+                    # any. Elements are (lswitch_name, list_of_acls).
+                    if len(aclr[1]) > 0:
+                        LOG.warning('Removing ACLs from OVN from Logical '
+                                    'Switch %s', aclr[0])
+                        txn.add(self.ovn_api.acl_del(aclr[0]))
+
+    def _sync_acls(self, ctx):
+        """Sync ACLs between neutron and NB when not using Port Groups.
 
         @param ctx: neutron_lib.context
         @type  ctx: object of type neutron_lib.context.Context
@@ -234,9 +386,6 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         @var   subnet_cache: cache for subnets
         @return: Nothing
         """
-        LOG.debug('ACL-SYNC: started @ %s' %
-                  str(datetime.now()))
-
         db_ports = {}
         for port in self.core_plugin.get_ports(ctx):
             db_ports[port['id']] = port
@@ -292,8 +441,21 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                         is_add_acl=False
                     ))
 
-        LOG.debug('ACL-SYNC: finished @ %s' %
-                  str(datetime.now()))
+    def sync_acls(self, ctx):
+        """Sync ACLs between neutron and NB.
+
+        @param ctx: neutron_lib.context
+        @type  ctx: object of type neutron_lib.context.Context
+        @return: Nothing
+        """
+        LOG.debug('ACL-SYNC: started @ %s', str(datetime.now()))
+
+        if self.ovn_api.is_port_groups_supported():
+            self._sync_acls_port_groups(ctx)
+        else:
+            self._sync_acls(ctx)
+
+        LOG.debug('ACL-SYNC: finished @ %s', str(datetime.now()))
 
     def _calculate_fips_differences(self, ovn_fips, db_fips):
         to_add = []
@@ -337,7 +499,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                       "sync routers and router ports")
             return
 
-        LOG.debug('OVN-NB Sync Routers and Router ports started @ %s' %
+        LOG.debug('OVN-NB Sync Routers and Router ports started @ %s',
                   str(datetime.now()))
 
         db_routers = {}
@@ -351,20 +513,26 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
             db_extends[router['id']]['fips'] = []
             if not router.get(l3.EXTERNAL_GW_INFO):
                 continue
-            gw_info = self._ovn_client._get_gw_info(ctx, router)
-            if gw_info.gateway_ip:
-                db_extends[router['id']]['routes'].append(
-                    {'destination': '0.0.0.0/0',
-                     'nexthop': gw_info.gateway_ip})
-            if gw_info.router_ip and utils.is_snat_enabled(router):
-                networks = (
-                    self._ovn_client._get_v4_network_of_all_router_ports(
-                        ctx, router['id']))
-                for network in networks:
-                    db_extends[router['id']]['snats'].append({
-                        'logical_ip': network,
-                        'external_ip': gw_info.router_ip,
-                        'type': 'snat'})
+            gateways = self._ovn_client._get_gw_info(ctx, router)
+            for gw_info in gateways:
+                prefix = (constants.IPv4_ANY if
+                          gw_info.ip_version == constants.IP_VERSION_4 else
+                          constants.IPv6_ANY)
+                if gw_info.gateway_ip:
+                    db_extends[router['id']]['routes'].append(
+                        {'destination': prefix,
+                         'nexthop': gw_info.gateway_ip})
+                if gw_info.ip_version == constants.IP_VERSION_6:
+                    continue
+                if gw_info.router_ip and utils.is_snat_enabled(router):
+                    networks = (
+                        self._ovn_client._get_v4_network_of_all_router_ports(
+                            ctx, router['id']))
+                    for network in networks:
+                        db_extends[router['id']]['snats'].append({
+                            'logical_ip': network,
+                            'external_ip': gw_info.router_ip,
+                            'type': 'snat'})
 
         fips = self.l3_plugin.get_floatingips(
             ctx, {'router_id': list(db_routers.keys())})
@@ -585,7 +753,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                                 logical_ip=nat['logical_ip'],
                                 external_ip=nat['external_ip'],
                                 type='snat'))
-        LOG.debug('OVN-NB Sync routers and router ports finished %s' %
+        LOG.debug('OVN-NB Sync routers and router ports finished %s',
                   str(datetime.now()))
 
     def _sync_subnet_dhcp_options(self, ctx, db_networks,
@@ -732,6 +900,12 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
             dhcp_ports = self.core_plugin.get_ports(ctx, filters=dict(
                 network_id=[net['id']],
                 device_owner=[constants.DEVICE_OWNER_DHCP]))
+
+            for port in dhcp_ports:
+                # Do not touch the Neutron DHCP agents ports
+                if utils.is_neutron_dhcp_agent_port(port):
+                    dhcp_ports.remove(port)
+
             if not dhcp_ports:
                 LOG.warning('Missing metadata port found in Neutron for '
                             'network %s', net['id'])
@@ -966,34 +1140,6 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
             # Add the ports to the default Port Group
             txn.add(self.ovn_api.pg_add_ports(pg_name, ports_ids))
 
-    def _create_subnets_port_group(self, ctx, db_ports):
-        filters = {'enable_dhcp': [1]}
-        db_subnets = {}
-        for subnet in self.core_plugin.get_subnets(ctx, filters=filters):
-            if subnet['ip_version'] == constants.IP_VERSION_6 and (
-                    subnet.get('ipv6_address_mode') == constants.IPV6_SLAAC):
-                continue
-            db_subnets[subnet['id']] = subnet
-
-        with self.ovn_api.transaction(check_error=True) as txn:
-            for subnet_id, subnet in db_subnets.items():
-                pg_name = utils.ovn_port_group_name(subnet_id)
-                if not self.ovn_api.get_port_group(pg_name):
-                    # If subnet Port Group doesn't exist yet, create it.
-                    txn.add(self.ovn_api.pg_add(pg_name, acls=[]))
-                    # Add ACLs to this Port Group.
-                    acls = acl_utils.add_acls_for_subnet_port_group(
-                        self.ovn_api, pg_name, subnet)
-                    for acl in acls:
-                        txn.add(self.ovn_api.pg_acl_add(**acl))
-            # Add ports to the relevant subnet Port Groups
-            for port in db_ports:
-                for ip in port['fixed_ips']:
-                    if ip['subnet_id'] in db_subnets:
-                        txn.add(self.ovn_api.pg_add_ports(
-                            utils.ovn_port_group_name(ip['subnet_id']),
-                                port['id']))
-
     def _create_sg_port_groups_and_acls(self, ctx, db_ports):
         # Create a Port Group per Neutron Security Group
         with self.ovn_api.transaction(check_error=True) as txn:
@@ -1015,14 +1161,15 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         # Groups and SG Rules to the new Port Groups implementation.
         # 1. Create the default drop Port Group and add all ports with port
         #    security enabled to it.
-        # 2. Create a Port Group for every subnet to allow DHCP traffic.
-        #    and add the relevant ports to them.
-        # 3. Create a Port Group for every existing Neutron Security Group and
+        # 2. Create a Port Group for every existing Neutron Security Group and
         #    add all its Security Group Rules as ACLs to that Port Group.
-        # 4. Delete all existing Address Sets in NorthBound database which
+        # 3. Delete all existing Address Sets in NorthBound database which
         #    correspond to a Neutron Security Group.
-        # 5. Delete all the ACLs in every Logical Switch (Neutron network).
-        if not self.ovn_api.is_port_groups_supported():
+        # 4. Delete all the ACLs in every Logical Switch (Neutron network).
+
+        # If Port Groups are not supported or we've already migrated, return
+        if (not self.ovn_api.is_port_groups_supported() or
+                not self.ovn_api.get_address_sets()):
             return
 
         LOG.debug('Port Groups Migration task started')
@@ -1036,7 +1183,6 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                     utils.is_port_security_enabled(port)]
 
         self._create_default_drop_port_group(db_ports)
-        self._create_subnets_port_group(ctx, db_ports)
         self._create_sg_port_groups_and_acls(ctx, db_ports)
         self._delete_address_sets(ctx)
         self._delete_acls_from_lswitches(ctx)
